// $Id: J4CDCDriftRegionHit.cc,v 1.1.1.2 2001/05/31 06:35:26 hoshina Exp $
//*************************************************************************
//* --------------------
//* J4CDCDriftRegionHit
//* --------------------
//* (Description)
//* 	Derivated concreate hit class for J4CDCDriftRegion.
//*	Mother class : J4VHit
//*     
//* (Update Record)
//*	2001/02/18  K.Hoshina	Original version.
//*************************************************************************

#include <iomanip.h>
#include <math.h>
#include "J4CDCDriftRegionHit.hh"
#include "G4Tubs.hh"

//=========================================================================
//---------------------
// Allocator
//---------------------

J4CDCDriftRegionHitAllocator J4CDCDriftRegionHit::fHitAllocator ;

//=========================================================================
//* constructor -----------------------------------------------------------

J4CDCDriftRegionHit::J4CDCDriftRegionHit(
		J4VDetectorComponent* detector,   // He is in "location" now
  	 	G4int		      cloneID,	  // CloneID of detector
		G4int		      trackID,	  // TrackID
  	 	G4ParticleDefinition* particle,	  // Particle
		G4double	      tof,	  // TOF 
		G4double 	      edep,	  // Energy Deposit
		G4double 	      totalE,	  // Total energy
		const G4ThreeVector&  momentum,	  // Momentum of perticle
 		const G4ThreeVector&  pre,	  // Pre-position of track
 		const G4ThreeVector&  pos )	  // Post-position of track
 	: J4VHit(detector, cloneID, trackID, particle, tof, edep, 
 		totalE, momentum, pre, pos)
 		
{
  fHitPosition = GetHitPosition();
}

//=========================================================================
//* destructor -----------------------------------------------------------

J4CDCDriftRegionHit::~J4CDCDriftRegionHit()
{
}

//=========================================================================
//* GetHitPosition --------------------------------------------------

G4ThreeVector J4CDCDriftRegionHit::GetHitPosition() const
{
  G4ThreeVector pre  = GetPrePosition();
  G4ThreeVector post = GetPostPosition();
  G4double x = (pre.x() + post.x()) / 2.0 ;
  G4double y = (pre.y() + post.y()) / 2.0 ;
  G4double z = (pre.z() + post.z()) / 2.0 ;
  
  G4ThreeVector hit(x, y, z);
  return hit;
}


//=========================================================================
//* Output ----------------------------------------------------------------

void J4CDCDriftRegionHit::Output(G4HCofThisEvent* HCTE)
{

  G4int wireNo  = GetDetectorComponent()->GetMyID();
  G4int layerNo = GetDetectorComponent()->GetMother()->GetMyID();

  G4String pid  = GetParticle()->GetParticleName();
  G4int    pdid = GetParticle()->GetPDGEncoding();   
  
  G4double dphi = ((G4Tubs*)GetDetectorComponent()->
  			GetSolid())->GetDeltaPhiAngle();
  G4double sphi = ((G4Tubs*)GetDetectorComponent()->
  			GetSolid())->GetStartPhiAngle();
  
  G4double rmin = ((G4Tubs*)GetDetectorComponent()->GetSolid())->
  			GetInnerRadius();  
  G4double rmax = ((G4Tubs*)GetDetectorComponent()->GetSolid())->
  			GetOuterRadius();  
  
  G4double hphi    = fHitPosition.phi();
  G4double cellphi = hphi - sphi;
  while(cellphi < 0)      cellphi += 2*M_PI;
  while(cellphi > 2*M_PI) cellphi -= 2*M_PI;   
  G4int cellID     = (G4int)(cellphi/dphi);
		
  G4double rw   = (rmax + rmin )*0.5;
  G4double wphi = (cellID-0.5) * dphi - sphi;
 
  G4double phirot = fCellRot.phiX();
  G4double hphirot = hphi + phirot;
  
  	
  G4double hwphi = hphirot - sphi -0.5*dphi;
  while(hwphi < 0)      hwphi += 2*M_PI;
  while(hwphi > 2*M_PI) hwphi -= 2*M_PI;   
  if(hwphi > M_PI)      hwphi =  2*M_PI - hwphi;
  
  G4double driftlen = rw * hwphi;
  
	
#if 1
  ofstream& ofs = GetOutputFileStream();
  if(! ofs.good()) {
    G4String errorMessage= "J4CDCDriftRegionHit::Output(): write error.";
    G4Exception(errorMessage);
  }  
  else
  {
     ofs << layerNo << " " << wireNo << " " << cellID << " " << GetTrackID() << " "
         << pdid << " " << dphi << " " << sphi << " "
         << wphi << " " << rw << " " << driftlen << " "
         << fHitPosition.x() << " " << fHitPosition.y() << " " << fHitPosition.z() << " "
         << GetMomentum().x() << " " << GetMomentum().y() << " "
         << GetMomentum().z() << " " << GetTotalEnergy() << " " 
         << GetEnergyDeposit() << " " << GetTof() <<  G4endl;
  }
#else
  G4cout << "SDname = " << GetDetectorComponent()->GetName() << " " 
  	 << "TrackID = " << GetTrackID() << " " 
  	 << "ParticleName = " << GetParticleName() << " "
  	 << "EnergyDeposit = " << GetEnergyDeposit() << " "
  	 << "TotalEnergy = " << GetTotalEnergy() << G4endl
  	 << "PreStepPoint(x,y,z) = " << GetPrePosition().x() << " "
  	 <<  GetPrePosition().y() << " " << GetPrePosition().z() << " "
  	 << "HitPoint(x,y,z) = " << fHitPosition.x() << " "
  	 << fHitPosition.y() << " "<<  fHitPosition.z() 
  	 << G4endl; 
#endif
}

//=========================================================================
//* Draw ------------------------------------------------------------------

void J4CDCDriftRegionHit::Draw()
{
}

//=========================================================================
//* Print -----------------------------------------------------------------

void J4CDCDriftRegionHit::Print()
{

  G4cout << setiosflags(ios::fixed);
  G4cout << setw(20) << GetDetectorComponentName() << " " << G4endl;
  G4cout << " track#=" << GetTrackID()
	 << " charge=" << GetCharge()
	 << " position(mm)= " << setprecision(2) 
	 << setw(8) << fHitPosition.x() *(1./mm) << " "
	 << setw(8) << fHitPosition.y() *(1./mm) << " "
	 << setw(8) << fHitPosition.z() *(1./mm) << " "
	 << " energy(GeV)= " << setprecision(2) 
	 << setw(6) << GetTotalEnergy() *(1./GeV) << " "
         << " TOF(ns)= " << setw(4) << setprecision(1) 
         << GetTof() /ns 
         << G4endl;  
}

